package cz.upol.jj1.lecture12;

import java.util.Arrays;
import java.util.Random;

/**
 * Simple matrix generator.
 * Elements are ints.
 * @author Unknown
 */
public class Matrix{

    private int[][] data;

    /**
      * @return Number of rows in matrix
     */
    public int getRowsCount(){
        return data.length;
    }

    /**
     *
     * @return Number of columns in matrix
     */
    public int getColsCount(){
        return data[0].length;
    }

    /**
     * Returns intern representation of array values.
     * Be careful leeks inner state.
     * @return array of values
     */
    protected int[][] getData(){
        return data;
    }

    /**
     *
     * @param row row index
     * @param col column index
     * @return value on the given indices
     */
    public int getValue(int row, int col){
        return data[row][col];
    }

    /**
     * Sets value on given indices to value
     * @param row row index
     * @param col column index
     * @param value value to use
     */
    public void setValue(int row, int col, int value){
        data[row][col] = value;
    }

    /**
     * Creates matrix with all elements equal to the given value
     * @param rows number of rows
     * @param columns number of columns
     * @param value value to intialize all elements with
     */
    public Matrix(int rows, int columns, int value){
        data = new int[rows][columns];
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                data[i][j] = value;
            }
        }
    }

    /**
     * Creates matrix with random elements between 1 and 100
     * @param rows number of rows
     * @param columns number of columns
     */
    public Matrix(int rows, int columns) {
        data = new int[rows][columns];
        Random random = new Random();
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                data[i][j] = random.nextInt(1, 100) ;
            }
        }
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        int rows = getRowsCount();
        if (rows == 0)
            return "";
        int columns = getColsCount();

        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                sb.append(data[i][j] + "  ");
            }
            sb.append(System.lineSeparator());
        }
        return sb.toString();
    }

    /**
     * Creates new matrix by adding first matrix and second matrix
     * @param first first matrix
     * @param second second matrix
     * @throws IllegalArgumentException if the arguments have different dimensions
     * @return sum of the given matrices
     */
    public static Matrix add(Matrix first, Matrix second){
        if(first.getRowsCount() != second.getRowsCount() || first.getColsCount() != second.getColsCount()){
            throw new IllegalArgumentException("Can not add matrices with different dimensions");
        }
        Matrix result = new Matrix(first.getRowsCount(), first.getColsCount(), 0);
        for(var i  = 0; i < first.getRowsCount(); i++){
            for(var j = 0; j < first.getColsCount(); j++){
                result.setValue(i, j, first.getValue(i, j) + second.getValue(i,j));
            }
        }
        return result;
    }


    /**
     * Creates new matrix by multiplying first matrix by second matrix
     * @param matrix1 first matrix
     * @param matrix2 second matrix
     * @throws IllegalArgumentException if the arguments have incompatible dimensions
     * @return Matrix product of the given matrices
     */
    public static Matrix multiply(Matrix matrix1, Matrix matrix2) {
        int rows = matrix1.getRowsCount();
        int columns2 = matrix2.getColsCount();
        int columns = matrix1.getColsCount();
        int rows2 = matrix2.getRowsCount();
        if(columns != rows2){
            throw new IllegalArgumentException("Illegal dimensions combinaton");
        }
        Matrix result = new Matrix(rows, columns, 0);
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns2; j++) {
                for (int k = 0; k < columns; k++) {
                    result.setValue(i, j, result.getValue(i,j) + matrix1.getValue(i, k) * matrix2.getValue(k, j));
                }
            }
        }
        return result;
    }

    /**
     * Matrix subtraction
     * @version beta
     */
    public static Matrix subtract(Matrix first, Matrix second){
        if(true)
            throw new UnsupportedOperationException("Not supported yet.");
        if(first.getRowsCount() != second.getRowsCount() || first.getColsCount() != second.getColsCount()){
            throw new IllegalArgumentException("Can not add matrices with different dimensions");
        }
        Matrix result = new Matrix(first.getRowsCount(), first.getColsCount(), 0);
        for(var i  = 0; i < first.getRowsCount(); i++){
            for(var j = 0; j < first.getColsCount(); j++){
                result.setValue(i, j, first.getValue(i, j) - second.getValue(i,j));
            }
        }
        return result;
    }

    /**
     * Two matrices are equal if and only if they have same dimensions and their respective elements are equal.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Matrix matrix = (Matrix) o;

        for(var i  = 0; i < getRowsCount(); i++){
            for(var j = 0; j < getColsCount(); j++){
                if(data[i][j] != matrix.getValue(i,j)){
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public int hashCode() {
        return Arrays.hashCode(data);
    }
}
